@@ -91,15 +91,15 @@ static
 globus_result_t
 ggvm_extract_cert_from_chain(
     gss_ctx_id_t                        context,
-    int                                 cert_index,
     X509 **                             out_cert)
 {
     globus_result_t                     result = GLOBUS_SUCCESS;
     OM_uint32                           major_status;
     OM_uint32                           minor_status;
     gss_buffer_set_t                    cert_chain_buffers = 0;
-    X509 *                              cert;
+    X509 *                              cert = NULL;
     const unsigned char *               ptr;
+    int                                 cert_index = 0;
 
     major_status = gss_inquire_sec_context_by_oid(
         &minor_status,
@@ -116,26 +116,48 @@ ggvm_extract_cert_from_chain(
         goto err;
     }
 
-    if(cert_chain_buffers->count <= cert_index)
+    while(!cert && cert_index < cert_chain_buffers->count)
     {
-        GLOBUS_GRIDMAP_CALLOUT_ERROR(
-            result,
-            GLOBUS_GRIDMAP_CALLOUT_GSSAPI_ERROR,
-            ("too few certs in chain"));
-        goto err;
+        globus_gsi_cert_utils_cert_type_t   cert_type;
+        
+        ptr = cert_chain_buffers->elements[cert_index].value;
+        cert = d2i_X509(
+            NULL,
+            GT_D2I_ARG_CAST &ptr,
+            cert_chain_buffers->elements[cert_index].length);
+        if(cert == NULL)
+        {
+            GLOBUS_GRIDMAP_CALLOUT_ERROR(
+                result,
+                GLOBUS_GRIDMAP_CALLOUT_GSSAPI_ERROR,
+                ("error reading cert chain"));
+            goto err;
+        }
+
+        result = globus_gsi_cert_utils_get_cert_type(cert, &cert_type);
+        if(result != GLOBUS_SUCCESS)
+        {
+            GLOBUS_GRIDMAP_CALLOUT_ERROR(
+                result,
+                GLOBUS_GRIDMAP_CALLOUT_GSSAPI_ERROR,
+                ("error searching for EEC in cert chain"));
+            goto err;
+        }
+
+        if(cert_type != GLOBUS_GSI_CERT_UTILS_TYPE_EEC)
+        {
+            X509_free(cert);
+            cert = NULL;
+            cert_index++;
+        }
     }
 
-    ptr = cert_chain_buffers->elements[cert_index].value;
-    cert = d2i_X509(
-        NULL,
-        GT_D2I_ARG_CAST &ptr,
-        cert_chain_buffers->elements[cert_index].length);
-    if(cert == NULL)
+    if(!cert)
     {
         GLOBUS_GRIDMAP_CALLOUT_ERROR(
             result,
             GLOBUS_GRIDMAP_CALLOUT_GSSAPI_ERROR,
-            ("can't extract cert from chain"));
+            ("no EEC found in cert chain"));
         goto err;
     }
 
@@ -294,6 +316,7 @@ ggvm_get_myproxy_userid(
     char *                              subject,
     char **                             userid,
     X509 *                              shared_user_cert,
+    STACK_OF(X509) *                    shared_user_chain,
     time_t                              shared_exp)
 {
     X509 *                              user_cert = NULL;
@@ -337,11 +360,40 @@ ggvm_get_myproxy_userid(
     /* extract user cert */
     if(shared_user_cert)
     {
-        user_cert = shared_user_cert;
+        globus_gsi_cert_utils_cert_type_t   cert_type;
+        
+        result = globus_gsi_cert_utils_get_cert_type(
+            shared_user_cert, &cert_type);
+        if(result != GLOBUS_SUCCESS)
+        {
+            GLOBUS_GRIDMAP_CALLOUT_ERROR(
+                result,
+                GLOBUS_GRIDMAP_CALLOUT_GSSAPI_ERROR,
+                ("error checking shared user cert type"));
+            goto error;
+        }
+
+        if(cert_type == GLOBUS_GSI_CERT_UTILS_TYPE_EEC)
+        {
+            user_cert = shared_user_cert;
+        }
+        else if(shared_user_chain)
+        {
+            result = globus_gsi_cert_utils_get_eec(
+                shared_user_chain, &user_cert);            
+        }
+        if(result != GLOBUS_SUCCESS || !user_cert)
+        {
+            GLOBUS_GRIDMAP_CALLOUT_ERROR(
+                result,
+                GLOBUS_GRIDMAP_CALLOUT_GSSAPI_ERROR,
+                ("EEC not found in shared user cert chain"));
+            goto error;
+        }
     }
     else
     {
-        result = ggvm_extract_cert_from_chain(context, 0, &user_cert);
+        result = ggvm_extract_cert_from_chain(context, &user_cert);
         if(result != GLOBUS_SUCCESS)
         {
             GLOBUS_GRIDMAP_CALLOUT_ERROR(
@@ -513,6 +565,7 @@ globus_gridmap_eppn_callout(
     int                                 rc;
     char *                              shared_user_buf = NULL;
     X509 *                              shared_user_cert = NULL;
+    STACK_OF(X509) *                    shared_user_chain = NULL;
     time_t                              shared_exp = 0;
     
     rc = globus_module_activate(GLOBUS_GSI_GSS_ASSIST_MODULE);
@@ -532,7 +585,11 @@ globus_gridmap_eppn_callout(
         shared_user_buf = va_arg(ap, char *);
         
         result = globus_gsi_cred_read_cert_buffer(
-            shared_user_buf, &tmp_cred_handle, &shared_user_cert, NULL, &subject);
+            shared_user_buf, 
+            &tmp_cred_handle, 
+            &shared_user_cert, 
+            &shared_user_chain, 
+            &subject);
         
         globus_gsi_cred_get_goodtill(tmp_cred_handle, &shared_exp);
         globus_gsi_cred_handle_destroy(tmp_cred_handle);
@@ -552,7 +609,7 @@ globus_gridmap_eppn_callout(
     }
 
     result = ggvm_get_myproxy_userid(
-        context, subject, &found_identity, shared_user_cert, shared_exp);
+        context, subject, &found_identity, shared_user_cert, shared_user_chain, shared_exp);
     if(result == GLOBUS_SUCCESS)
     {
         if(desired_identity && strcmp(found_identity, desired_identity) != 0)
@@ -626,6 +683,10 @@ error:
     {
         X509_free(shared_user_cert);
     }
+    if(shared_user_chain)
+    {
+        sk_X509_free(shared_user_chain);
+    }
 
     globus_module_deactivate(GLOBUS_GRIDMAP_CALLOUT_ERROR_MODULE);
     globus_module_deactivate(GLOBUS_GSI_GSSAPI_MODULE);
